import { ChannelBlock, HTTPChannel, HTTPGuild, HTTPUser } from '..';
import { EventResolvable } from '../util/Types';
import GuildBlock from './blocks/GuildBlock';
import UserBlock from './blocks/UserBlock';
import ClientUser from './ClientUser';
import Intents from './Intents';
import Websocket from './websocket/Websocket';
export default class Client {
    /**
     * Whether or not this client is valid.
     */
    private valid;
    /**
     * The devtools instance for this client. Only set if
     * the `@birbjs/devtools` package is installed.
     */
    devtools: any;
    /**
     * The token of the client.
     *
     * ⚠️ **DON'T SHARE YOUR TOKEN!** There is no reason
     * anyone should need it, with the exception of your
     * development team. ANYONE with your token can gain
     * FULL and UNRESTRICTED ACCESS to the entirety of your
     * bot, INCLUDING ANY PERMISSIONS IT HAS BEEN GRANTED.
     */
    token: string;
    /**
     * The client's websocket.
     */
    ws: Websocket;
    /**
     * The client's Discord user.
     */
    me: ClientUser | null;
    /**
     * The client's Shard if sharding is enabled. See the
     * [docs](https://birb.js.org/addons/sharding) for more
     * information.
     */
    shard: any;
    /**
     * The client's options.
     */
    options: {
        intents: Intents;
        debug: boolean;
    };
    /**
     * Direct access to the client's rest API endpoints.
     */
    readonly api: {
        channels: typeof HTTPChannel;
        guilds: typeof HTTPGuild;
        users: typeof HTTPUser;
    };
    /**
     * The client's guild cache.
     */
    guilds: GuildBlock;
    /**
     * The client's channel cache.
     */
    channels: ChannelBlock;
    /**
     * The client's user cache.
     */
    users: UserBlock;
    /**
     * A private object containing all event listeners.
     */
    private events;
    /**
     * The client is the entrypoint to Birb.JS. If you'd
     * like to learn how to use Birb.JS, head over to our
     * [docs](https://birb.js.org) or have a quick look at
     * our [examples](https://birb.js.org/start).
     *
     * @param {object} options The client options.
     * @param {string} options.intents The [Intents](https://birb.js.org/start/intents) to use.
     * @param {boolean} [options.debug=false] Whether or not to enable the client's debug mode. Requires the `@birbjs/devtools` package.
     */
    constructor(options: {
        intents: Intents;
        debug?: boolean;
    });
    /**
     * Add an event listener.
     *
     * @param {EventResolvable | string} event The event name.
     * @param {Function} listener The function to call when the event is emitted.
     * @returns {void} Voids once the event listener is added.
     */
    listen(event: EventResolvable | string, callback: Function): void;
    /**
     * Undind an event listener.
     *
     * @param {EventResolvable | string} event The event name to unbind.
     * @returns {void} Voids once the event listener is removed.
     */
    unbind(event: EventResolvable | string): void;
    /**
     * Emit an event.
     *
     * @param {EventResolvable | string} event The event name.
     * @param {any} data The event data to pass to the listener.
     * @returns {any} The response of the listener's method. `undefined` if not bound.
     */
    emit(event: EventResolvable | string, ...args: any[]): any;
    /**
     * Initilize a new addon to the client.
     *
     * @param {Function} fn The addon's main function.
     * @returns {any} The updated client.
     */
    add(fn: Function): Client;
    /**
     * Connect to the Discord gateway.
     *
     * ⚠️ **DON'T SHARE YOUR TOKEN!** There is no reason
     * anyone should need it, with the exception of your
     * development team. ANYONE with your token can gain
     * FULL and UNRESTRICTED ACCESS to the entirety of your
     * bot, INCLUDING ANY PERMISSIONS IT HAS BEEN GRANTED.
     *
     * @param {string} token Your bot's token.
     * @returns {void} Voids once the client has started (not connected).
     */
    connect(token: string): void;
    /**
     * Emit a log in debug mode.
     *
     * @param {any} message The log message.
     * @returns {void} Voids once the log has been emitted.
     */
    debug(...message: any[]): void;
    /**
     * Emit a warning in debug mode.
     *
     * @param {any} message The warning message.
     * @returns {void} Voids once the log has been emitted.
     */
    warn(...message: any[]): void;
    /**
     * Log a packet received from the gateway.
     *
     * @param {any} message The packet data.
     * @returns {void} Voids once the log has been emitted.
     */
    private logReceive;
    /**
     * Log a packet sent to the gateway.
     *
     * @param {any} message The packet data.
     * @returns {void} Voids once the log has been emitted.
     */
    private logSend;
    /**
     * Log a HTTP event.
     *
     * @param {any} message The event data.
     * @returns {void} Voids once the log has been emitted.
     */
    private logHttp;
    /**
     * Only registers the provided event if it isn't
     * already registered.
     *
     * @param {string} event The event name.
     * @param {Function} callback The function to call when the event is emitted.
     * @returns {void} Voids once the event has been registered.
     */
    private listenIfNotRegistered;
    /**
     * Invalidate the client, forcing the user to restart
     * the process to use it. Once called, this cannot be
     * undone (mostly because it would defeat the entire
     * purpose of this function).
     *
     * @returns {void} Voids once the client has been invalidated.
     */
    invalidate(): void;
}
