"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseUser_1 = __importDefault(require("./BaseUser"));
const HTTPUser_1 = __importDefault(require("./http/HTTPUser"));
const OptionError_1 = __importDefault(require("../errors/OptionError"));
class ClientUser extends BaseUser_1.default {
    /**
     * Represents the user that is logged in to the client.
     *
     * @param {Client} client The client this user belongs to.
     * @param {any} data The data of this user.
     */
    constructor(client, data) {
        super(client, data);
        this.bot = true;
        this.system = false;
        Object.freeze(this.bot);
        Object.freeze(this.system);
        this.build(data);
    }
    /**
     * Builds the ClientUser.
     *
     * @param {any} data The data of this user.
     * @returns {ClientUser} The ClientUser.
     */
    build(data) {
        this.username = data.username ?? 'Unknown';
        this.discriminator = data.discriminator ?? '0000';
        this.tag = `${this.username}#${this.discriminator}`;
        this.flags = data.public_flags ?? 0;
        if ('avatar' in data) {
            this.avatar = data.avatar;
        }
        if ('banner' in data) {
            this.banner = data.banner;
        }
        if ('accent_color' in data) {
            this.accentColor = data.accent_color;
        }
        return this;
    }
    /**
     * Updates the user's presence.
     *
     * @param {object} [options] The options to use when updating the status.
     * @param {ActivityStatus} [options.status='online'] The status to set.
     * @param {boolean} [options.afk=false] Whether or not the user is AFK.
     * @param {object} [options.activity=null] The activity to set.
     * @param {string} options.activity.name The name of the activity.
     * @param {ActivityType} options.activity.type The URL of the activity.
     * @returns {void} Voids once the packet is sent to Discord.
     */
    updatePresence(options = {}) {
        if (!options.activity)
            options.activity = null;
        this.client.ws.send({
            op: 3,
            d: {
                status: options.status ?? 'online',
                afk: options.afk ?? false,
                activities: options.activity ? [options.activity] : null,
                since: options.afk ? Date.now() : null,
            },
        });
    }
    /**
     * Set the username of the user.
     *
     * @param {string} username The new username.
     * @returns {Promise<void>} Resolves once the username is set.
     */
    async setName(name) {
        if (name === undefined) {
            throw new OptionError_1.default('name [at 0] must be provided');
        }
        if (typeof name !== 'string') {
            throw new OptionError_1.default('name [at 0] must be a string with a length of at least 1 character');
        }
        await HTTPUser_1.default.modifyCurrent(this.client, { username: name });
    }
}
exports.default = ClientUser;
