import { NSFWLevel, MFALevel, NotificationLevel, VerificationLevel, ExplicitContentFilterLevel } from '../util/Constants';
import RoleBlock from './blocks/RoleBlock';
import Client from './Client';
import GuildMemberBlock from './blocks/GuildMemberBlock';
import { ChannelResolvable } from '../util/Types';
import GuildChannelBlock from './blocks/GuildChannelBlock';
export default class Guild {
    /**
     * The client this Guild belongs to.
     */
    client: Client;
    /**
     * The ID of this Guild.
     */
    readonly id: string;
    /**
     * The name of this Guild.
     */
    name: string;
    /**
     * The guild's icon hash.
     * **Note:** This is not the same as an icon URL. Use
     * the `getIconURL` method to get the icon URL of this
     * guild.
     */
    icon: string | null;
    /**
     * Whether or not Discord has marked this guild as Not
     * Safe For Wumpus (explicit).
     */
    nsfw: boolean;
    /**
     * The guild's discovery splash hash.
     * **Note:** This is not the same as a splash URL. Use
     * the `getSplashURL` method to get the splash URL of
     * this guild.
     */
    discoverySplash: string | null;
    /**
     * The guild's banner hash.
     * **Note:** This is not the same as an banner URL.
     * Use the `getBannerURL` method to get the icon URL of
     * this guild.
     */
    banner: string | null;
    /**
     * The guild's description.
     */
    description: string | null;
    /**
     * The verification level of this guild.
     */
    verificationLevel: VerificationLevel;
    /**
     * The default notification level of this guild.
     */
    defaultNotifications: NotificationLevel;
    /**
     * Whether or not this guild is available to this
     * client. **You should check this is set to `true`
     * before performing actions on guilds.**
     */
    available: boolean;
    /**
     * The ID of this guild's system channel where join
     * notifications, boost messages, etc. are sent.
     */
    systemChannelId: string | null;
    /**
     * The ID of the owner of this guild.
     */
    ownerId: string;
    /**
     * Whether or not this guild is deemed 'large' (at
     * which point Discord will not automatically send a
     * list of members).
     */
    large: boolean;
    /**
     * The ID of the guild's AFK channel.
     */
    afkChannelId: string | null;
    /**
     * The setting of the explicit content filter for this
     * guild.
     */
    explicitContentFilter: ExplicitContentFilterLevel;
    /**
     * The maximum amount of members allowed in this guild.
     * Reach out to [Discord support](https://dis.gd/support)
     * if you're getting close to this limit and need it
     * increased.
     */
    maxMembers: number | null;
    /**
     * The preferred locale (language) for this Guild.
     */
    preferredLocale: string | null;
    /**
     * The multi-factor authentication requirement level
     * for this guild's staff members.
     */
    mfaLevel: MFALevel;
    /**
     * The vanity invite code this guild owns. `null` if it
     * doesn't have one.
     */
    vanityCode: string | null;
    /**
     * The Not Safe For Wumpus (explicit) level Discord has
     * assigned to this guild. Discord uses this to block
     * NSFW servers on iOS.
     */
    nsfwLevel: NSFWLevel;
    /**
     * The maximum amount of video channel members.
     */
    maxVideoChannelUsers: number | null;
    /**
     * The approximate amount of members in this guild.
     */
    approximateMemberCount: number | null;
    /**
     * The approximate amount of online members in this
     * guild.
     */
    approximatePresenceCount: number | null;
    /**
     * The amount of members who have used Discord Nitro
     * to boost this guild (or have purchased a Nitro boost
     * on its own for this guild).
     */
    boosterCount: number | null;
    /**
     * The splash hash of this guild.
     * **Note:** This is not the same as a splash URL.
     * Use the `getSplashURL` method to get the splash URL
     * of this guild.
     */
    splash: string | null;
    /**
     * The member count of this guild.
     */
    memberCount: number | null;
    /**
     * The boost tier of this guild.
     */
    boostTier: number;
    /**
     * The channel ID of the guild's rules or guidelines
     * channel.
     */
    rulesChannelId: string | null;
    /**
     * The amount of seconds to wait before moving AFK
     * members to the AFK voice channel.
     */
    afkTimeout: number;
    /**
     * The roles this guild has.
     */
    roles: RoleBlock;
    /**
     * The members this guild has.
     */
    members: GuildMemberBlock;
    /**
     * The channels this guild has.
     */
    channels: GuildChannelBlock;
    /**
     * A Guild represents a Discord server.
     *
     * @param {Client} client The client that this guild belongs to.
     * @param {any} data The data for this guild.
     */
    constructor(client: Client, data: any);
    /**
     * Builds this guild with the given data.
     *
     * @param {any} data The data to build this guild with.
     * @returns {Guild} This guild.
     */
    private build;
    /**
     * Change this Guild's name.
     *
     * @param {String} name The new name for the Guild.
     * @param {String} [reason] The reason for changing the Guild's name.
     * @returns {Promise<void>} A Promise that voids when the Guild's name has been changed.
     */
    setName(name: string, reason?: string): Promise<void>;
    /**
     * Change this Guild's verification level.
     *
     * @param {VerificationLevel} level The new verification level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setVerificationLevel(level: VerificationLevel, reason?: string): Promise<void>;
    /**
     * Change this Guild's default notifications level.
     *
     * @param {NotificationLevel} level The new notification level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setDefaultNotifications(level: NotificationLevel, reason?: string): Promise<void>;
    /**
     * Change this Guild's explicit content filter level.
     *
     * @param {ExplicitContentFilterLevel} level The new explicit content filter level for the Guild.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setExplicitContentFilter(level: ExplicitContentFilterLevel, reason?: string): Promise<void>;
    /**
     * Set this Guild's AFK timeout channel.
     *
     * @param {ChannelResolvable} channel The AFK timeout channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setAfkChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Change this Guild's AFK timeout.
     *
     * @param {number} seconds The AFK timeout in seconds.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setAfkTimeout(seconds: number, reason?: string): Promise<void>;
    /**
     * Set this Guild's system messages channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setSystemChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set this Guild's rules channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setRulesChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set this Guild's public updates channel.
     *
     * @param {ChannelResolvable} channel The channel.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setPublicUpdatesChannel(channel: ChannelResolvable, reason?: string): Promise<void>;
    /**
     * Set the Guild's preferred locale.
     *
     * @param {String} locale The new locale.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setPreferredLocale(locale: string, reason?: string): Promise<void>;
    /**
     * Set the Guild's description.
     *
     * @param {String} description The new description.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setDescription(description: string, reason?: string): Promise<void>;
    /**
     * Toggle the Guild's booster progress bar.
     *
     * @param {boolean} toggle Whether or not to show the booster progress bar.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    setBoostProgressBar(toggle: number, reason?: string): Promise<void>;
    /**
     * Modify the Guild.
     *
     * @param {Object} data The data to send to the Discord API.
     * @param {String} [reason] The reason for modifying this Guild.
     * @returns {Promise<void>} A Promise that voids when the Guild has been changed.
     */
    modify(data: Object, reason?: string): Promise<void>;
    /**
     * Leave the Guild.
     *
     * @returns {Promise<void>} A Promise that voids when the Guild has been left.
     */
    leave(): Promise<void>;
    /**
     * Convert this Guild into a a string (the name).
     *
     * @returns {string} The name of this Guild.
     */
    toString(): string;
    /**
     * Set the Guild's data to the cache.
     *
     * @returns {Guild} This Guild instance.
     */
    private set;
}
