import Guild from './Guild';
import BaseUser from './BaseUser';
import GuildMemberRoleBlock from './blocks/GuildMemberRoleBlock';
import Client from './Client';
export default class GuildMember {
    /**
     * The client that this member belongs to.
     */
    client: Client;
    /**
     * The user ID of this member.
     */
    readonly id: string;
    /**
     * The guild this member is in.
     */
    guild: Guild;
    /**
     * The user related to this member.
     */
    user: BaseUser | null;
    /**
     * The original user object of this member.
     */
    protected originalUser: any;
    /**
     * Whether or not this member has been fully resolved.
     */
    protected full: boolean;
    /**
     * The nickname of this member.
     */
    nick: string | null;
    /**
     * The avatar hash for this member.
     * **Note:** This is not the same as an avatar URL. Use
     * the `displayAvatarURL` method to get the avatar URL
     * of this user.
     */
    avatar: string | null;
    /**
     * The date this member joined the guild.
     */
    joinedAt: Date | null;
    /**
     * Whether or not this member is deafened in this
     * guild's voice channels. If `true`, the member won't
     * be able to hear anything said in voice channels.
     */
    deafened: boolean;
    /**
     * Whether or not this member is muted in this guild's
     * voice channels. If `true`, the member won't be able
     * to say anything in voice channels.
     */
    muted: boolean;
    /**
     * The date until this member leaves timeout mode. This
     * will be `null` if the member is not in timeout mode.
     */
    timedOutUntil: Date | null;
    /**
     * Whether or not this member is pending membership
     * screening in this guild. Until membership screening
     * is passed, the member may not interact with the
     * guild in any way.
     */
    pendingMembershipScreening: boolean;
    /**
     * The roles this member has.
     */
    roles: GuildMemberRoleBlock;
    /**
     * A GuildMember represents a member of a Discord
     * guild (or "server").
     *
     * @param {Client} client The client that this member belongs to.
     * @param {any} data The data of this member.
     * @param {Guild} guild The guild this member is in.
     */
    constructor(client: Client, data: any, guild: Guild);
    /**
     * Build this GuildMember.
     *
     * @param data The data of this GuildMember.
     */
    private build;
    /**
     * Check if this GuildMember is in timeout.
     *
     * @returns {boolean} Whether this GuildMember is in timeout.
     */
    isInTimeout(): boolean;
    /**
     * Set this GuildMember's muted status.
     *
     * @param {boolean} deafen The new muted status.
     * @param {String} [reason] The reason for modifying this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    setMute(mute: boolean, reason?: string): Promise<void>;
    /**
     * Set this GuildMember's deafened status.
     *
     * @param {boolean} deafen The new deafened status.
     * @param {String} [reason] The reason for modifying this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    setDeafen(deafen: boolean, reason?: string): Promise<void>;
    /**
     * Set this GuildMember's nickname.
     *
     * @param {number} nickname The new nickname.
     * @param {String} [reason] The reason for modifying this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    setNick(nickname: string, reason?: string): Promise<void>;
    /**
     * Kick the GuildMember.
     *
     * @param {String} [reason] The reason for kicking this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    kick(reason?: string): Promise<void>;
    /**
     * Ban the GuildMember.
     *
     * @param {number} [daysToPrune=0] The amount of days of messages to prune.
     * @param {String} [reason] The reason for banning this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    ban(daysToPrune: number, reason?: string): Promise<void>;
    /**
     * Modify the GuildMember.
     *
     * @param {Object} data The data to send to the Discord API.
     * @param {String} [reason] The reason for modifying this GuildMember.
     * @returns {Promise<void>} A Promise that voids when the GuildMember has been changed.
     */
    modify(data: Object, reason?: string): Promise<void>;
    /**
     * Forces the client to wait until the member has been
     * fully resolved. If the member is in the cache, the
     * promise will resolve immediately. However, if the
     * client will fetch the member from the Discord API.
     *
     * @returns {Promise<GuildMember>} A Promise that resolves when the member has been fully resolved.
     */
    _waitForFull(): Promise<GuildMember>;
    /**
     * Sets the GuildMember to the cache.
     *
     * @returns {GuildMember} The GuildMember.
     */
    private set;
}
