import Guild from './Guild';
import BaseUser from './BaseUser';
import Client from './Client';
import GuildMember from './GuildMember';
import TextBasedChannel from './TextBasedChannel';
import { MessageContent } from '../util/Types';
import { MessageTypes } from '../util/Constants';
import MessageAttachment from './message/MessageAttachment';
export default class Message {
    /**
     * The client this message belongs to.
     */
    client: Client;
    /**
     * The ID of the message.
     */
    readonly id: string;
    /**
     * The text content of the message.
     */
    content: string | null;
    /**
     * The webhook ID if this message was sent by a Discord
     * webhook.
     */
    webhookId: string | null;
    /**
     * The guild this message was sent in, if any.
     */
    guild: Guild | null;
    /**
     * The author of this message.
     */
    author: BaseUser | null;
    /**
     * The member that sent this message if this message
     * was sent in a guild.
     */
    member: GuildMember | null;
    /**
     * Whether or not this is a system message (e.g. a
     * welcome message).
     */
    system: boolean;
    /**
     * The channel this message was sent in.
     */
    channel: TextBasedChannel;
    /**
     * Any attachments that were sent with this message.
     */
    attachments: MessageAttachment[];
    /**
     * The type of message.
     */
    type: keyof typeof MessageTypes;
    /**
     * This message's bitfield flags.
     */
    flags: number;
    /**
     * Base user data sent by the API/GW.
     */
    private baseAuthor;
    /**
     * A Message represents a message in any channel on
     * Discord.
     *
     * @param {Client} client The client this message belongs to.
     * @param {any} data The data of the message.
     */
    constructor(client: Client, data: any);
    /**
     * Build the message from the data.
     *
     * @param {any} data The data of the message.
     * @returns {Message} The message.
     */
    private build;
    /**
     * Crosspost this message to channels that are
     * following the channel this message was sent in.
     *
     * @returns {Promise<Message>} The crossposted message.
     */
    crosspost(): Promise<Message>;
    /**
     * Delete this message.
     *
     * @returns {Promise<void>} A promise that voids when the message is deleted.
     */
    delete(): Promise<void>;
    /**
     * Send an inline reply to this message.
     *
     * @param {MessageContent} content The content of the reply.
     * @returns {Promise<Message>} The sent message.
     */
    reply(message: MessageContent): Promise<Message>;
    /**
     * Pin this message to the channel it was sent in.
     *
     * @param {string} [reason] The reason for pinning this message.
     * @returns {Promise<Message>} The pinned message.
     */
    pin(reason?: string): Promise<Message>;
    /**
     * Unpin this message from the channel it was sent in.
     *
     * @param {string} [reason] The reason for unpinning this message.
     * @returns {Promise<Message>} The unpinned message.
     */
    unpin(reason?: string): Promise<Message>;
    /**
     * Start a thread from this message.
     *
     * @param {string} name The name of the thread.
     * @param {object} [options] The options for the thread.
     * @param {number} [options.autoArchiveDelay] The amount of time to wait before archiving the thread.
     * @param {number} [options.slowmode] The amount of time (in seconds) to set slowmode to for this thread.
     * @param {string} [options.reason] The reason for creating the thread.
     * @returns {Promise<void>} A promise that voids when the thread is created.
     * @todo Make this return a ThreadChannel instead of voiding.
     */
    startThread(name: string, options?: {
        autoArchiveDelay?: number;
        slowmode?: number;
        reason?: string;
    }): Promise<void>;
    /**
     * Supress the embeds of this message.
     *
     * @returns {Promise<Message>} The message.
     */
    supressEmbeds(): Promise<Message>;
    /**
     * Edit this message. Can only be performed if the
     * message was sent by this client.
     *
     * @param {MessageContent} content The new content of the message.
     * @returns {Promise<Message>} The edited message.
     */
    edit(message: MessageContent): Promise<Message>;
    /**
     * Send a raw API request to edit this message.
     *
     * @param {any} data The data to send.
     * @returns {Promise<Message>} The edited message.
     */
    modify(data: any): Promise<Message>;
    /**
     * Parse data into a message.
     *
     * @param {any} data The data to parse.
     * @returns {Promise<Message>} The parsed message.
     */
    private parse;
    /**
     * Wait for the author of the message to be resolved.
     *
     * @returns {Promise<Message>} The message with the resolved author.
     */
    private waitForAuthor;
    /**
     * Set the message to the cache.
     *
     * @returns {Message} The message.
     */
    private set;
    /**
     * Build an API message.
     *
     * @param {MessageContent} content The content of the message.
     * @param {boolean} [edit] Whether or not this is an edit.
     * @returns {any} The built message.
     */
    protected static buildApiMessage(data: MessageContent, edit?: boolean): any;
}
