"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const HTTPChannel_1 = __importDefault(require("./http/HTTPChannel"));
const Role_1 = __importDefault(require("./Role"));
const Constants_1 = require("../util/Constants");
const User_1 = __importDefault(require("./User"));
const MessageAttachment_1 = __importDefault(require("./message/MessageAttachment"));
const MessageEmbed_1 = __importDefault(require("./message/embed/MessageEmbed"));
class Message {
    /**
     * A Message represents a message in any channel on
     * Discord.
     *
     * @param {Client} client The client this message belongs to.
     * @param {any} data The data of the message.
     */
    constructor(client, data) {
        /**
         * The client this message belongs to.
         */
        this.client = null;
        /**
         * The text content of the message.
         */
        this.content = null;
        /**
         * The webhook ID if this message was sent by a Discord
         * webhook.
         */
        this.webhookId = null;
        /**
         * The guild this message was sent in, if any.
         */
        this.guild = null;
        /**
         * The author of this message.
         */
        this.author = null;
        /**
         * The member that sent this message if this message
         * was sent in a guild.
         */
        this.member = null;
        /**
         * Whether or not this is a system message (e.g. a
         * welcome message).
         */
        this.system = false;
        /**
         * The channel this message was sent in.
         */
        this.channel = null;
        /**
         * Any attachments that were sent with this message.
         */
        this.attachments = [];
        /**
         * The type of message.
         */
        this.type = null;
        /**
         * This message's bitfield flags.
         */
        this.flags = 0;
        /**
         * Base user data sent by the API/GW.
         */
        this.baseAuthor = null;
        this.client = client;
        this.id = data.id;
        this.build(data);
    }
    /**
     * Build the message from the data.
     *
     * @param {any} data The data of the message.
     * @returns {Message} The message.
     */
    build(data) {
        this.baseAuthor = data.author || null;
        if ('content' in data) {
            this.content = data.content;
        }
        if ('attachments' in data) {
            this.attachments = MessageAttachment_1.default['fromApiMessage'](data.attachments);
        }
        if ('webhook_id' in data) {
            this.webhookId = data.webhook_id;
        }
        if ('guild_id' in data) {
            this.guild = this.client.guilds.cache.get(data.guild_id) || null;
            if (this.guild)
                this.channel = this.guild.channels.cache.get(data.channel_id);
        }
        if ('member' in data) {
            try {
                data.member.user = this.baseAuthor;
                data.member.id = this.baseAuthor.id;
                this.member = this.guild?.members.resolve(this.baseAuthor.id, data.member) || null;
            }
            catch (err) { }
        }
        if ('type' in data) {
            if (Constants_1.SystemMessageTypes[data.type])
                this.system = true;
            this.type = Constants_1.MessageTypes[data.type];
        }
        if (!this.webhookId && !this.system) {
            if (data.author.id == this.client.me?.id) {
                this.author = this.client.me;
            }
            else {
                this.author = this.client.users.resolve(data.author.id, data.author ?? null) || null;
            }
        }
        return this;
    }
    /**
     * Crosspost this message to channels that are
     * following the channel this message was sent in.
     *
     * @returns {Promise<Message>} The crossposted message.
     */
    async crosspost() {
        return this.parse(await HTTPChannel_1.default.crossPostMessage(this.client, this.channel.id, this.id));
    }
    /**
     * Delete this message.
     *
     * @returns {Promise<void>} A promise that voids when the message is deleted.
     */
    async delete() {
        await HTTPChannel_1.default.deleteMessage(this.client, this.channel.id, this.id);
    }
    /**
     * Send an inline reply to this message.
     *
     * @param {MessageContent} content The content of the reply.
     * @returns {Promise<Message>} The sent message.
     */
    async reply(message) {
        await this.waitForAuthor();
        let data = Message.buildApiMessage(message);
        data.message_reference = {
            message_id: this.id,
            channel_id: this.channel.id,
            fail_if_not_exists: false,
        };
        let msg = await HTTPChannel_1.default.createMessage(this.client, this.channel.id, data);
        return new Message(this.client, msg).waitForAuthor();
    }
    /**
     * Pin this message to the channel it was sent in.
     *
     * @param {string} [reason] The reason for pinning this message.
     * @returns {Promise<Message>} The pinned message.
     */
    async pin(reason) {
        await HTTPChannel_1.default.pinMessage(this.client, this.channel.id, this.id, reason);
        return this;
    }
    /**
     * Unpin this message from the channel it was sent in.
     *
     * @param {string} [reason] The reason for unpinning this message.
     * @returns {Promise<Message>} The unpinned message.
     */
    async unpin(reason) {
        await HTTPChannel_1.default.unpinMessage(this.client, this.channel.id, this.id, reason);
        return this;
    }
    /**
     * Start a thread from this message.
     *
     * @param {string} name The name of the thread.
     * @param {object} [options] The options for the thread.
     * @param {number} [options.autoArchiveDelay] The amount of time to wait before archiving the thread.
     * @param {number} [options.slowmode] The amount of time (in seconds) to set slowmode to for this thread.
     * @param {string} [options.reason] The reason for creating the thread.
     * @returns {Promise<void>} A promise that voids when the thread is created.
     * @todo Make this return a ThreadChannel instead of voiding.
     */
    async startThread(name, options = {}) {
        await HTTPChannel_1.default.startThreadWithMessage(this.client, this.channel.id, this.id, {
            name: name,
            auto_archive_duration: options.autoArchiveDelay ?? null,
            rate_limit_per_user: options.slowmode ?? 0,
        }, options.reason);
    }
    /**
     * Supress the embeds of this message.
     *
     * @returns {Promise<Message>} The message.
     */
    async supressEmbeds() {
        return this.modify({
            flags: this.flags | Constants_1.MessageFlags.SUPPRESS_EMBEDS,
        });
    }
    /**
     * Edit this message. Can only be performed if the
     * message was sent by this client.
     *
     * @param {MessageContent} content The new content of the message.
     * @returns {Promise<Message>} The edited message.
     */
    async edit(message) {
        return this.modify(Message.buildApiMessage(message, true));
    }
    /**
     * Send a raw API request to edit this message.
     *
     * @param {any} data The data to send.
     * @returns {Promise<Message>} The edited message.
     */
    async modify(data) {
        let res = await HTTPChannel_1.default.editMessage(this.client, this.channel.id, this.id, data, this);
        this.build(res);
        return this.set();
    }
    /**
     * Parse data into a message.
     *
     * @param {any} data The data to parse.
     * @returns {Promise<Message>} The parsed message.
     */
    async parse(data) {
        this.build(data);
        await this.waitForAuthor();
        return this.set();
    }
    /**
     * Wait for the author of the message to be resolved.
     *
     * @returns {Promise<Message>} The message with the resolved author.
     */
    async waitForAuthor() {
        if (!this.author && !this.webhookId && !this.baseAuthor.system) {
            if (this.client.me && this.baseAuthor.id === this.client.me.id) {
                this.author = this.client.me;
            }
            else {
                this.author = await this.client.users.fetch(this.baseAuthor.id);
            }
        }
        return this;
    }
    /**
     * Set the message to the cache.
     *
     * @returns {Message} The message.
     */
    set() {
        if (this.guild)
            this.guild.channels.cache.get(this.channel.id).messages.cache.set(this.id, this);
        return this;
    }
    /**
     * Build an API message.
     *
     * @param {MessageContent} content The content of the message.
     * @param {boolean} [edit] Whether or not this is an edit.
     * @returns {any} The built message.
     */
    static buildApiMessage(data, edit = false) {
        if (typeof data === 'string') {
            return { content: data };
        }
        if (edit) {
            return {
                content: data.content ?? undefined,
                embeds: data.embeds?.map(e => e instanceof MessageEmbed_1.default ? e.format() : e) ?? undefined,
                tts: data.tts ?? undefined,
                nonce: data.nonce ?? undefined,
                attachments: data.attachments ?? undefined,
                allowed_mentions: {
                    parse: data.allowedMentions?.parse ?? undefined,
                    users: data.allowedMentions?.users?.map((u) => User_1.default['toIdOnly'](u)) ?? undefined,
                    roles: data.allowedMentions?.roles?.map((r) => Role_1.default['toIdOnly'](r)) ?? undefined,
                    replied_user: data.mentionRepliedUser ?? undefined,
                },
            };
        }
        else {
            return {
                content: data.content ?? null,
                embeds: data.embeds?.map(e => e instanceof MessageEmbed_1.default ? e.format() : e) ?? [],
                tts: data.tts ?? false,
                nonce: data.nonce ?? null,
                attachments: data.attachments ?? [],
                allowed_mentions: {
                    parse: data.allowedMentions?.parse ?? null,
                    users: data.allowedMentions?.users?.map((u) => User_1.default['toIdOnly'](u)) ?? null,
                    roles: data.allowedMentions?.roles?.map((r) => Role_1.default['toIdOnly'](r)) ?? null,
                    replied_user: data.mentionRepliedUser ?? false,
                },
            };
        }
    }
}
exports.default = Message;
