"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const GuildError_1 = __importDefault(require("../errors/GuildError"));
const Color_1 = __importDefault(require("../util/Color"));
const HTTPGuild_1 = __importDefault(require("./http/HTTPGuild"));
const OptionError_1 = __importDefault(require("../errors/OptionError"));
const RolePermissionsBlock_1 = __importDefault(require("./blocks/RolePermissionsBlock"));
class Role {
    /**
     * A Role represents a role in a guild on Discord.
     *
     * @param {Client} client The client that this Role belongs to.
     * @param {any} data The data of this Role.
     * @param {Guild} guild The guild this Role belongs to.
     */
    constructor(client, data, guild) {
        /**
         * The client that this Role belongs to.
         */
        this.client = null;
        /**
         * The guild this Role belongs to.
         */
        this.guild = null;
        /**
         * The name of this Role.
         */
        this.name = null;
        /**
         * The color of this Role as a hex color string.
         */
        this.color = null;
        /**
         * Whether or not this Role is hoisted on the member
         * list.
         */
        this.hoist = null;
        /**
         * The icon hash of this Role.
         */
        this.icon = null;
        /**
         * Whether or not this Role is managed by Discord,
         * another application or a bot. If so, it cannot be
         * deleted.
         */
        this.managed = null;
        /**
         * Whether or not anyone can mention this Role.
         */
        this.mentionable = null;
        /**
         * The permissions of this Role.
         */
        this.permissions = null;
        /**
         * The position of this Role.
         */
        this.position = null;
        /**
         * The unicode emoji of this Role.
         */
        this.unicodeEmoji = null;
        if (typeof data.id !== 'string') {
            throw new GuildError_1.default('invalid role data provided');
        }
        this.client = client;
        this.id = data.id;
        this.guild = guild;
        this.build(data);
    }
    /**
     * Build this Role.
     *
     * @param {any} data The data to build this Role with.
     * @returns {Role} This Role.
     */
    build(data) {
        if ('name' in data) {
            this.name = data.name;
        }
        if ('color' in data) {
            let hex = Color_1.default.intToHex(data.color);
            this.color = `#${hex === '00' ? '000000' : hex}`;
        }
        if ('hoist' in data) {
            this.hoist = data.hoist;
        }
        if ('icon' in data) {
            this.icon = data.icon;
        }
        if ('managed' in data) {
            this.managed = data.managed;
        }
        if ('mentionable' in data) {
            this.mentionable = data.mentionable;
        }
        if ('permissions' in data) {
            this.permissions = new RolePermissionsBlock_1.default(this.client, this, parseInt(data.permissions) ?? 0);
        }
        if ('position' in data) {
            this.position = data.position;
        }
        if ('unicode_emoji' in data) {
            this.unicodeEmoji = data.unicode_emoji;
        }
        return this;
    }
    /**
     * Delete this Role.
     *
     * @param {string} [reason] The reason for deleting this Role.
     * @returns {Promise<void>} A Promise that voids when the Role has been deleted.
     */
    async delete(reason) {
        await HTTPGuild_1.default.deleteRole(this.client, this.guild.id, this.id, reason);
    }
    /**
     * Set the name of this Role.
     *
     * @param {string} name The name to set this Role to.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setName(name, reason) {
        if (name === undefined) {
            throw new OptionError_1.default('name [at 0] must be provided');
        }
        if (typeof name !== 'string') {
            throw new OptionError_1.default('name [at 0] must be a string with a length of at least 1 character');
        }
        return this.modify({ name }, reason);
    }
    /**
     * Set the position of this Role.
     *
     * @param {number} position The position to set this Role to.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setPosition(position, reason) {
        if (position === undefined) {
            throw new OptionError_1.default('position [at 0] must be provided');
        }
        if (typeof position !== 'number' || position < 0) {
            throw new OptionError_1.default('position [at 0] must be a positive number');
        }
        return HTTPGuild_1.default.modifyRolePositions(this.client, this.guild.id, [{ id: this.id, position }], reason);
    }
    /**
     * Move this Role's position.
     *
     * @param {number} velocity The velocity to apply to this Role's position. Provide a positive integer to move this Role up, and a negative integer to move this Role down.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    move(velocity, reason) {
        if (velocity === undefined) {
            throw new OptionError_1.default('velocity [at 0] must be provided');
        }
        if (typeof velocity !== 'number') {
            throw new OptionError_1.default('velocity [at 0] must be a number');
        }
        let position = this.position != null ? this.position + velocity : 1;
        return this.setPosition(position, reason);
    }
    /**
     * Set the color of this Role.
     *
     * @param {number | string} color The new color for this Role. Accepts HEX color codes (e.g. ffffff for white) or an RGB integer.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setColor(color, reason) {
        if (color === undefined) {
            throw new OptionError_1.default('color [at 0] must be provided');
        }
        if (typeof color == 'number') {
            return this.modify({ color }, reason);
        }
        else if (typeof color == 'string') {
            return this.modify({ color: Color_1.default.hexToInt(color) }, reason);
        }
        else {
            throw new OptionError_1.default('color [at 0] must be a number or string');
        }
    }
    /**
     * Set if this Role can be mentioned.
     *
     * @param {boolean} mentionable Whether or not this Role can be mentioned.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setIsMentionable(mentionable, reason) {
        if (mentionable === undefined) {
            throw new OptionError_1.default('mentionable [at 0] must be provided');
        }
        if (typeof mentionable !== 'boolean') {
            throw new OptionError_1.default('mentionable [at 0] must be a boolean');
        }
        return this.modify({ mentionable }, reason);
    }
    /**
     * Set if this members of this Role should be hoisted
     * on the member list.
     *
     * @param {boolean} hoisted Whether or not this Role should be hoisted on the member list.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    setIsHoisted(hoisted, reason) {
        if (hoisted === undefined) {
            throw new OptionError_1.default('hoisted [at 0] must be provided');
        }
        if (typeof hoisted !== 'boolean') {
            throw new OptionError_1.default('hoisted [at 0] must be a boolean');
        }
        return this.modify({ hoist: hoisted }, reason);
    }
    /**
     * Modify this Role.
     *
     * @param {Object} data The data to modify this Role with.
     * @param {string} [reason] The reason for modifying this Role.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    async modify(data, reason) {
        let res = await HTTPGuild_1.default.modifyRole(this.client, this.guild.id, this.id, data, reason);
        this.build(res);
        return this.set();
    }
    /**
     * Convert this Role into a mention (string).
     *
     * @returns {string} A string that mentions this role.
     */
    toString() {
        return `<@&${this.id}>`;
    }
    /**
     * Handle updates from the Discord Gateway.
     *
     * @param {Client} client The client that received the update.
     * @param {any} data The data to handle.
     * @param {Guild} guild The Guild this Role belongs to.
     * @returns {Role} A reference to this Role.
     */
    static handleUpdate(client, data, guild) {
        let role = guild.roles.cache.get(data.id);
        if (role) {
            role.build(data);
            guild.roles.cache.set(data.id, role);
            return role;
        }
        role = new Role(client, data, guild);
        guild.roles.cache.set(data.id, role);
        return role;
    }
    /**
     * Set the Role's data to the cache.
     *
     * @returns {Role} This Role instance.
     * @private
     */
    set() {
        this.client.guilds.cache.set(this.id, this);
        return this;
    }
    /**
     * Convert a RoleResolvable into a role ID (string).
     *
     * @param {RoleResolvable} role The Role to convert.
     * @returns {string} The ID of the Role.
     */
    static toIdOnly(role) {
        if (typeof role === 'string') {
            return role;
        }
        else if (role instanceof Role) {
            return role.id;
        }
        else {
            throw new TypeError(`Expected a string or Role, received ${typeof role}`);
        }
    }
}
exports.default = Role;
