import Guild from './Guild';
import Client from './Client';
import TextBasedChannel from './TextBasedChannel';
export default class TextChannel extends TextBasedChannel {
    /**
     * A TextChannel represents any guild text channel on
     * Discord.
     *
     * @param {Client} client The client this channel belongs to.
     * @param {any} data The data of this channel.
     * @param {object} [options] The options of this channel.
     * @param {number} [options.maxSize=null] The maximum size of the cache.
     * @param {number} [options.maxAge=null] The maximum age (in seconds) of the values in the cache.
     * @param {number} [options.checkInterval=60] The interval (in seconds) to check for old values in the cache.
     * @param {boolean} [options.removeOldest=true] Whether to remove the oldest values when the cache is full.
     * @param {Guild} [guild] The guild this channel belongs to.
     */
    constructor(client: Client, data: any, options?: {
        maxSize?: number;
        maxAge?: number;
        checkInterval?: number;
        removeOldest?: boolean;
    }, guild?: Guild);
    /**
     * Build the TextChannel.
     *
     * @param {any} data The data of this channel.
     */
    private build;
    /**
     * Set the name of this channel.
     *
     * @param {string} name The name of this channel.
     * @param {string} [reason] The reason for changing the name.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    setName(name: string, reason?: string): Promise<TextChannel>;
    /**
     * Set the position of this channel.
     *
     * @param {number} position The position of this channel.
     * @param {string} [reason] The reason for changing the position.
     * @returns
     */
    setPosition(position: number, reason?: string): Promise<TextChannel>;
    /**
     * Set the topic of this channel.
     *
     * @param {string} topic The topic of this channel.
     * @param {string} [reason] The reason for changing the topic.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    setTopic(topic: string, reason?: string): Promise<TextChannel>;
    /**
     * Set whether or not this channel is Not Safe For
     * Wumpus (explict).
     *
     * @param {boolean} nsfw Whether or not this channel is explicit.
     * @param {string} [reason] The reason for changing the NSFW status.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    setNSFW(nsfw: boolean, reason?: string): Promise<TextChannel>;
    /**
     * Sets the slowmode of this channel.
     *
     * @param {number} seconds The number of seconds of slowmode.
     * @param {string} [reason] The reason for changing the slowmode.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    setSlowmode(seconds: number, reason?: string): Promise<TextChannel>;
    /**
     * Set the auto-archive timeout of this channel.
     *
     * @param {number} minutes The auto-archive timeout of this channel in minutes.
     * @param {string} [reason] The reason for changing the auto-archive timeout.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    setAutoArchive(minutes: number, reason?: string): Promise<TextChannel>;
    /**
     * Send a raw modify request to the Discord API.
     *
     * @param {any} data The data to send.
     * @param {string} [reason] The reason for modifying this channel.
     * @returns {Promise<TextChannel>} This TextChannel instance.
     */
    modify(data: any, reason?: string): Promise<TextChannel>;
    /**
     * Set the TextChannel's data to the cache.
     *
     * @returns {TextChannel} This TextChannel instance.
     */
    protected set(): TextChannel;
}
