import { UserResolvable } from '../util/Types';
import BaseUser from './BaseUser';
import Client from './Client';
import Message from './Message';
export default class User extends BaseUser {
    /**
     * Raw data on this user's DM channel.
     */
    protected dmChannel: any;
    /**
     * A User represents any Discord user.
     *
     * @param {Client} client The client that instantiated this User.
     * @param {any} data The data for this User.
     */
    constructor(client: Client, data: any);
    /**
     * Build this User.
     *
     * @param {any} data The data to build this User with.
     * @returns {User} This User instance.
     */
    private build;
    /**
     * Generate a link to the user's avatar.
     *
     * @param {Object} [options] The options to use when generating the URL.
     * @param {string} [options.format=png] The image format to use for the URL.
     * @param {boolean} [options.dynamic=false] Whether or not a GIF should be returned autoamtically if the user's avatar is animated.
     * @returns {string} The URL to the user's avatar.
     */
    getAvatar(options?: {
        format?: 'png' | 'webp' | 'jpg' | 'gif';
        dynamic?: boolean;
    }): string;
    /**
     * Generate a link to the user's avatar.
     *
     * @param {Object} [options] The options to use when generating the URL.
     * @param {string} [options.format=png] The image format to use for the URL.
     * @param {boolean} [options.dynamic=false] Whether or not a GIF should be returned autoamtically if the user's banner is animated.
     * @returns {string} The URL to the user's banner.
     */
    getBanner(options?: {
        format?: 'png' | 'webp' | 'jpg' | 'gif';
        dynamic?: boolean;
    }): string;
    /**
     * Determine if the user's avatar is default.
     *
     * @returns {boolean} Whether or not the user's avatar is default.
     */
    hasDefaultAvatar(): boolean;
    /**
     * Fetch this user again.
     *
     * @param {Object} [options] The options to use when fetching the user.
     * @param {boolean} [options.shouldCache=true] Whether or not to cache the result.
     * @param {boolean} [options.bypassCache=true] Whether or not to bypass the cache.
     * @returns {Promise<User>} A promise that resolves with the updated user.
     */
    fetch(options?: {
        shouldCache?: boolean | undefined;
        bypassCache?: boolean | undefined;
    }): Promise<User>;
    /**
     * Send a message to the user.
     *
     * @param {string} [content] The content of the message.
     * @returns {Promise<Message>} A promise that resolves with the new message.
     */
    send(content: string): Promise<Message>;
    /**
     * Convert this User into a mention (string).
     *
     * @returns {string} A string that mentions this user.
     */
    toString(): string;
    /**
     * Set the User's data to the cache.
     *
     * @returns {User} This User instance.
     * @private
     */
    private set;
    /**
     * Either fetch a User from the cache or return a
     * PartialUser instance to avoid fetching from the API.
     *
     * @param {Client} client The client to use.
     * @param {Object} data The data on the user (`id` is required).
     * @returns {User | PartialUser} The User or PartialUser.
     */
    private static retrieveOrBuildPartial;
    /**
     * Converts a UserResolvable to a user ID (string).
     *
     * @param {UserResolvable} user The user to convert.
     * @returns {string} The user ID.
     */
    protected static toIdOnly(user: UserResolvable): string;
}
