"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CachedBlock_1 = __importDefault(require("./CachedBlock"));
const BaseUser_1 = __importDefault(require("../BaseUser"));
const GuildMember_1 = __importDefault(require("../GuildMember"));
const HTTPGuild_1 = __importDefault(require("../http/HTTPGuild"));
const CGCache_1 = __importDefault(require("../cache/CGCache"));
class GuildMemberBlock extends CachedBlock_1.default {
    /**
     * A GuildMemberBlock stores guild member data.
     *
     * @param {Client} client The client instance.
     * @param {Guild} guild The guild that the block is for.
     * @param {any} [options] The Cache options.
     */
    constructor(client, guild, options) {
        super(client, new CGCache_1.default(client, guild, options));
        /**
         * The guild that the block is for.
         */
        this.guild = null;
        this.guild = guild;
    }
    /**
     * Fetch a guild member from the cache or the Discord\
     * API if it is not in the cache.
     *
     * @param {string} userId The user's ID.
     * @param {object} [options] The options to pass to the API.
     * @param {boolean} [options.shouldCache=true] Whether or not to cache the member.
     * @param {boolean} [options.bypassCache=false] Whether to force the request to the API, bypassing the cache.
     * @returns {Promise<GuildMember>} The guild member.
     */
    async fetch(userId, options) {
        options = options || {};
        options.shouldCache = options.shouldCache ?? true;
        options.bypassCache = options.bypassCache ?? false;
        if (!options.bypassCache) {
            const cached = this.cache.get(userId);
            if (cached)
                return cached;
        }
        let member = await HTTPGuild_1.default.getMember(this.client, this.guild.id, userId);
        member = new GuildMember_1.default(this.client, member, this.guild);
        if (options.shouldCache)
            member._set();
        return member;
    }
    /**
     * Resolve a user to a GuildMember. Returns null if the
     * member is not cached.
     *
     * @param {UserResolvable} user The member to resolve.
     * @param {any} [def] The default value to return if the member is not cached.
     * @returns {GuildMember | null} The member. `null` if the member is not cached.
     */
    resolve(user, def) {
        if (user instanceof BaseUser_1.default) {
            return this.cache.get(user.id) ?? user;
        }
        else if (typeof user === 'string') {
            if (def)
                return this.cache.get(user) ?? new GuildMember_1.default(this.client, def, this.guild);
            else
                return this.cache.get(user) ?? null;
        }
        else {
            return null;
        }
    }
}
exports.default = GuildMemberBlock;
