"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const PermissionsBlock_1 = __importDefault(require("./PermissionsBlock"));
const BitsBlock_1 = __importDefault(require("./BitsBlock"));
class RolePermissionsBlock extends PermissionsBlock_1.default {
    /**
     * A RolePermissionsBlock stores role permissions data.
     *
     * @param {Client} client The client instance.
     * @param {Role} role The Role this PermissionsBlock is for.
     * @param {...number[]} flags The permissions to set.
     */
    constructor(client, role, ...flags) {
        super(client, ...flags);
        /**
         * The Role this PermissionsBlock is for.
         */
        this.role = null;
        this.role = role;
    }
    /**
     * Grant the role the specified permissions.
     *
     * @param {Permissions.FLAGS | number[] | number} permissions The permissions to grant.
     * @param {Object} options The options for the action.
     * @param {string} [options.reason] The reason for the action.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    async grant(permissions, options = {}) {
        let bits = this.bits.clone();
        if (typeof permissions == 'number') {
            bits.add(permissions);
        }
        else {
            for (let i = 0; i < permissions.length; ++i)
                bits.add(permissions[i]);
        }
        return await this.set(bits.flags, options);
    }
    /**
     * Revoke the specified permissions from the Role.
     *
     * @param {Permissions.FLAGS | number[] | number} permissions The permissions to revoke.
     * @param {Object} options The options for the action.
     * @param {string} [options.reason] The reason for the action.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    async revoke(permissions, options = {}) {
        let bits = this.bits.clone();
        if (typeof permissions == 'number') {
            bits.remove(permissions);
        }
        else {
            for (let i = 0; i < permissions.length; ++i)
                bits.remove(permissions[i]);
        }
        return await this.set(bits.flags, options);
    }
    /**
     * Set the permissions for the Role.
     *
     * @param {Permissions.FLAGS | number[] | number} permissions The permissions to set.
     * @param {Object} options The options for the action.
     * @param {string} [options.reason] The reason for the action.
     * @returns {Promise<Role>} A Promise that resolves to the modified Role.
     */
    async set(permissions, options = {}) {
        let bits = new BitsBlock_1.default(0);
        if (typeof permissions == 'number') {
            bits.set(permissions);
        }
        else {
            for (let i = 0; i < permissions.length; ++i)
                bits.add(permissions[i]);
        }
        if (bits.flags == this.bits.flags)
            return this.role;
        return await this.role.modify({ permissions: bits.flags }, options.reason);
    }
}
exports.default = RolePermissionsBlock;
