"use strict";
/*
 * Copyright (c) 2021, knokbak and contributors.
 *
 * The Birb.JS Project: https://birb.js.org
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const CachedBlock_1 = __importDefault(require("./CachedBlock"));
const ClientUser_1 = __importDefault(require("../ClientUser"));
const HTTPUser_1 = __importDefault(require("../http/HTTPUser"));
const User_1 = __importDefault(require("../User"));
const BaseUser_1 = __importDefault(require("../BaseUser"));
const CCache_1 = __importDefault(require("../cache/CCache"));
class UserBlock extends CachedBlock_1.default {
    /**
     * A UserBlock stores user data.
     *
     * @param {Client} client The client instance.
     * @param {any} [options] The Cache options.
     */
    constructor(client, options) {
        super(client, new CCache_1.default(client, options));
    }
    /**
     * Fetches this Client's user.
     *
     * @returns {Promise<User>} The Client's user.
     */
    async fetchMe() {
        let user = await HTTPUser_1.default.getCurrent(this.client);
        user = new ClientUser_1.default(this.client, user);
        this.cache.set(user.id, user);
        return user;
    }
    /**
     * Fetches a user from the Cache or the Discord API if
     * the user is not cached.
     *
     * @param {string} userId The ID of the user to fetch.
     * @param {object} [options] The options to use.
     * @param {boolean} [options.shouldCache=true] Whether to cache the user.
     * @param {boolean} [options.bypassCache=false] Whether to bypass the cache and fetch the user directly from the API.
     * @returns {Promise<User>} The user.
     */
    async fetch(userId, options) {
        options = options || {};
        options.shouldCache = options.shouldCache ?? true;
        options.bypassCache = options.bypassCache ?? false;
        if (!options.bypassCache) {
            const cached = this.cache.get(userId);
            if (cached)
                return cached;
        }
        let user = await HTTPUser_1.default.get(this.client, userId);
        user = new User_1.default(this.client, user);
        if (options.shouldCache)
            user['set']();
        return user;
    }
    /**
     * Resolves a UserResolvable to a user if the user is
     * in the cache.
     *
     * @param {UserResolvable} user The user to resolve.
     * @param {any} [def] The default value to build the user if the user is not cached.
     * @returns {User | null} The user. `null` if the user is not cached.
     */
    resolve(user, def) {
        if (user instanceof BaseUser_1.default)
            return this.cache.get(user.id) ?? user;
        else if (typeof user === 'string') {
            if (def)
                return this.cache.get(user) ?? new User_1.default(this.client, def);
            else
                return this.cache.get(user) ?? null;
        }
        else
            return null;
    }
}
exports.default = UserBlock;
