import Pair from '../../util/Pair';
import Client from '../Client';
export default abstract class Cache {
    /**
     * The Client that initialized this cache.
     */
    abstract client: Client;
    /**
     * The Cache's map (where the data is stored).
     */
    private cache;
    /**
     * The cache's options.
     *
     * @type {object} The options.
     * @property {number} [maxSize=null] The maximum size of the cache.
     * @property {number} [maxAge=null] The maximum age (in seconds) of the values in the cache.
     * @property {number} [checkInterval=60] The interval (in seconds) to check for old values in the cache.
     * @property {boolean} [removeOldest=true] Whether to remove the oldest values when the cache is full.
     */
    private options;
    /**
     * A Cache stores values in memory.
     *
     * @param {object} [options] The options for the cache.
     * @param {number} [options.maxSize=null] The maximum size of the cache.
     * @param {number} [options.maxAge=null] The maximum age (in seconds) of the values in the cache.
     * @param {number} [options.checkInterval=60] The interval (in seconds) to check for old values in the cache.
     * @param {boolean} [options.removeOldest=true] Whether to remove the oldest values when the cache is full.
     */
    constructor(options?: {
        maxSize?: number;
        maxAge?: number;
        checkInterval?: number;
        removeOldest?: boolean;
    });
    /**
     * Get a value from the cache. If the key is not found
     * in the cache, `null` is returned.
     *
     * @param {string} key The key to get the value for.
     * @returns {any | null} The value for the key, or `null` if the key is not found.
     */
    get(key: string): any | null;
    /**
     * Find a value in the cache. If the key is not found
     * in the cache, `null` is returned.
     *
     * @param {string} key The key to get the value for.
     * @returns {any | null} The value for the key, or `null` if the key is not found.
     */
    find(fn: ((e: any) => boolean)): any | null;
    /**
     * Filter values in the cache. If the key is not found
     * in the cache, an empty array is returned.
     *
     * @param {string} key The key to get the value for.
     * @returns {any[]} The value for the key, or [] if the key is not found.
     */
    filter(fn: ((e: any) => boolean)): any[];
    /**
     * Set a value in the cache.
     *
     * @param {string} key The key of the entry.
     * @param {any} value The value to set it to.
     * @returns {Cache} The updated cache.
     */
    set(key: string, value: any, options?: any): Cache;
    /**
     * Check if the cache contains a key.
     *
     * @param {string} key The key of the entry.
     * @returns {boolean} `true` if the key is found, `false` otherwise.
     */
    has(key: string): boolean;
    /**
     * Remove a key from the cache.
     *
     * @param {string} key The key of the entry.
     * @returns {Cache} The updated cache.
     */
    remove(key: string): Cache;
    /**
     * Get the amount of entries in the cache.
     *
     * @returns {number} The size of the cache.
     */
    size(): number;
    /**
     * Get an object of the entries in the cache. The
     * key is the key of the entry, and the value is
     * the value of the entry.
     *
     * @returns {{ [key: string]: any }} The array.
     */
    entries(): {
        [key: string]: any;
    };
    /**
     * Get an array of the entries in the cache as `Pair`s.
     *
     * @returns {any[]} The array.
     */
    arrayPair(): any[];
    /**
     * Get an array of the entries in the cache.
     *
     * @returns {Pair<string, any>[]} The array.
     */
    array(): Pair<string, any>[];
    /**
     * Clear the cache and its contents.
     *
     * @returns {Cache} The updated cache.
     */
    clear(): Cache;
    /**
     * Make space in the cache for a new entry. You
     * **probably shouldn't** run this method directly
     * unless you know **EXACTLY** what you're doing.
     *
     * @returns {void} Voids when completed.
     */
    makeSpace(options?: any): void;
    /**
     * A method called to strip unwanted properties from an
     * entry before it is added to the cache.
     *
     * @param {any} data The data to check and strip.
     * @returns {any} The updated data.
     */
    abstract strip(data: any): any;
    /**
     * A method called to rebuild data that has been
     * stripped.
     *
     * @param {any} data The data to check and rebuild.
     * @returns {any} The rebuilt data.
     */
    abstract rebuild(data: any): any;
}
