import BaseComponent from '../BaseComponent';
import EmbedAuthor from './EmbedAuthor';
import EmbedField from './EmbedField';
import EmbedFooter from './EmbedFooter';
import EmbedMedia from './EmbedMedia';
export default class MessageEmbed extends BaseComponent {
    /**
     * The title of this embed.
     */
    title?: string;
    /**
     * The description of this embed.
     */
    description?: string;
    /**
     * The URL of this embed (makes the title clickable).
     */
    url?: string;
    /**
     * The timestamp of this embed (shows in the footer).
     */
    timestamp?: Date;
    /**
     * The color of this embed as a hex string.
     */
    color?: string;
    /**
     * The footer of this embed.
     */
    footer?: EmbedFooter;
    /**
     * The image in this embed.
     */
    image?: EmbedMedia;
    /**
     * The thumbnail for this embed.
     */
    thumbnail?: EmbedMedia;
    /**
     * The video for this embed (usually not settable).
     */
    video?: EmbedMedia;
    /**
     * The author of this embed.
     */
    author?: EmbedAuthor;
    /**
     * This embed's fields.
     */
    fields: EmbedField[];
    /**
     * The MessageEmbed class represents a message embed on
     * Discord. A message embed is a rich embed that can
     * include a lot of formatted data (they can also have
     * their color set which is quite nice).
     */
    constructor();
    /**
     * Set the title of this embed.
     *
     * @param {string} title The new title of this embed.
     * @returns {MessageEmbed} This embed.
     */
    setTitle(title: string): MessageEmbed;
    /**
     * Set the description of this embed.
     *
     * @param {string} description The new description of this embed.
     * @returns {MessageEmbed} This embed.
     */
    setDescription(description: string): MessageEmbed;
    /**
     * Set the URL of this embed.
     *
     * @param {string} url The new URL of this embed.
     * @returns {MessageEmbed} This embed.
     */
    setUrl(url: string): MessageEmbed;
    /**
     * Set the timestamp of this embed.
     *
     * @param {Date | number} timestamp The new timestamp of this embed.
     * @returns {MessageEmbed} This embed.
     */
    setTimestamp(timestamp: Date | number): MessageEmbed;
    /**
     * Set the HEX color of this embed.
     *
     * @param {string} color The new HEX color of this embed.
     * @returns {MessageEmbed} This embed.
     */
    setColor(color: string): MessageEmbed;
    /**
     * Set the footer of this embed.
     *
     * @param {string} text The text of the footer.
     * @param {string} [iconUrl] A public-facing URL to an image for the footer.
     * @returns {MessageEmbed} This embed.
     */
    setFooter(text: string, iconUrl?: string): MessageEmbed;
    /**
     * Set the image for this embed.
     *
     * @param {string} url A public-facing URL to an image for the embed.
     * @param {number} [height] An optional height for the image.
     * @param {number} [width] An optional width for the image.
     * @returns {MessageEmbed} This embed.
     */
    setImage(url: string, height?: number, width?: number): MessageEmbed;
    /**
     * Set the thumbnail for this embed.
     *
     * @param {string} url A public-facing URL to an image for the embed.
     * @param {number} [height] An optional height for the image.
     * @param {number} [width] An optional width for the image.
     * @returns {MessageEmbed} This embed.
     */
    setThumbnail(url: string, height?: number, width?: number): MessageEmbed;
    /**
     * Set the video for this embed.
     *
     * @param {string} url A public-facing URL to a video for the embed.
     * @param {number} [height] An optional height for the video.
     * @param {number} [width] An optional width for the video.
     * @returns {MessageEmbed} This embed.
     */
    setVideo(url: string, height?: number, width?: number): MessageEmbed;
    /**
     * Set the author of this embed.
     *
     * @param {string} name The name of the author.
     * @param {string} [url] A public-facing URL for the author.
     * @param {string} [iconUrl] A public-facing URL to an image for the author.
     * @returns {MessageEmbed} This embed.
     */
    setAuthor(name: string, url?: string, iconUrl?: string): MessageEmbed;
    /**
     * Add a field to this embed.
     *
     * @param {string} name The name of the field.
     * @param {string} value The field's value.
     * @param {boolean} [inline=false] Whether or not this field should be displayed inline.
     * @returns {MessageEmbed} This embed.
     */
    addField(name: string, value: string, inline?: boolean): MessageEmbed;
    /**
     * Add multiple fields to this embed.
     *
     * @param {Object[]} fields The fields to add to this embed.
     * @param {string} fields.name The name of the field.
     * @param {string} fields.value The field's value.
     * @param {boolean} [fields.inline=false] Whether or not the field should be displayed inline.
     * @returns {MessageEmbed} This embed.
     */
    addFields(fields: {
        name: string;
        value: string;
        inline?: boolean;
    }[]): MessageEmbed;
    /**
     * Remove a field from this embed.
     *
     * @param {number} index The array index of the field to remove.
     * @returns {MessageEmbed} This embed.
     */
    removeField(index: number): MessageEmbed;
    /**
     * Set the fields of this embed.
     *
     * @param {Object[]} fields The new fields of this embed.
     * @param {string} fields.name The name of the field.
     * @param {string} fields.value The field's value.
     * @param {boolean} [fields.inline=false] Whether or not the field should be displayed inline.
     * @returns {MessageEmbed} This embed.
     */
    setFields(fields: {
        name: string;
        value: string;
        inline?: boolean;
    }[]): MessageEmbed;
    /**
     * Format this class into an API-acceptable object.
     *
     * @returns {any} The formatted object.
     */
    format(): any;
}
