import { Status } from '../../util/Constants';
import Client from '../Client';
export default class InternalWebsocket {
    /**
     * The client that owns this websocket.
     */
    client: Client;
    /**
     * The websocket's currently set URL.
     */
    url: URL;
    /**
     * The current Discord gateway URL.
     */
    domain: string;
    /**
     * The current status of the websocket.
     */
    status: Status;
    /**
     * The type of encoding to use. If `erlpack` is
     * installed (`npm install erlpack`), `etf` encoding
     * will be used. Otherwise, normal `json` encoding will
     * be used.
     */
    encoding: string;
    /**
     * The internal erlpack buffer.
     */
    protected buffer: any;
    /**
     * The InternalWebsocket class is a class that handles
     * the annoying stuff regarding websockets connected to
     * Discord (like compression and packing).
     *
     * @param {Client} client The client that owns this websocket.
     * @param {string} domain The current Discord gateway URL.
     */
    constructor(client: Client, domain: string);
    /**
     * Attempts to process a packet by using ZLib to let
     * Discord send packets in chunks. If ZLib is not yet
     * installed (`npm install zlib-sync`), this method
     * will just pass the data through for unpacking
     * (`this.unpack()`).
     *
     * @param {any} data The packet's data.
     * @returns {any} Hopefully, the packet's JSON form.
     */
    protected processPacket(data: any): any;
    /**
     * Unpacks a packet using the current encoding. If the
     * encoding is set to `etf`, this method will use the
     * `zlib-sync` (`npm install zlib-sync`) library to
     * unpack its `etf` data. Otherwise, parses the data to
     * normal JSON.
     *
     * @param {any} data The packet's data.
     * @param {string} [type] The packet's encoding.
     * @returns {any} Hopefully, the packet's JSON form.
     */
    protected unpack(data: any, type?: string): any;
    /**
     * Packs data for sending to Discord. If the encoding
     * is set to `etf`, this method will use the
     * `zlib-sync` (`npm install zlib-sync`) library to
     * pack its `etf` data. Otherwise, parses the JSON data
     * to a string.
     *
     * @param {any} data The data to pack.
     * @returns {any} Hopefully, the data's packed form.
     */
    protected pack(data: any): any;
    /**
     * Initializes ZLib if supported.
     *
     * @returns {void} Voids once ZLib is initialized.
     */
    protected init(): void;
    /**
     * Generates the gateway URL.
     *
     * @returns {URL} The gateway URL.
     */
    protected generateURL(): URL;
}
