/// <reference types="node" />
import Client from '../Client';
import InternalWebsocket from './InternalWebsocket';
import { WebSocket as WS } from 'ws';
import { Status } from '../../util/Constants';
export default class Websocket extends InternalWebsocket {
    /**
     * The websocket instance from the `ws` package.
     */
    ws: WS;
    /**
     * The current latency (in milliseconds) between the
     * client and the Discord gateway.
     */
    latency: number;
    /**
     * The guild IDs that the client is expecting to
     * receive from the Discord gateway.
     */
    expectedGuilds: Set<string>;
    /**
     * The time the last heartbeat was sent.
     */
    lastHeartbeat: number;
    /**
     * Whether or not the last heartbeat was acknowledged
     * by the Discord gateway.
     */
    lastHeartbeatAcked: boolean;
    /**
     * The interval to send heartbeats.
     */
    heartbeatInterval: number | null;
    /**
     * The websocket's scheduler loop.
     */
    schedulerLoop: NodeJS.Timer | null;
    /**
     * The last sequence identifier received from the
     * Discord gateway.
     */
    lastSequenceIdentifier: number | null;
    /**
     * The session identifier from the Discord gateway.
     */
    sessionIdentifier: string | null;
    /**
     * Whether or not to prevent the websocket from
     * attempting to reconnect. Used to prevent loops.
     */
    private doNotReconnect;
    /**
     * The time the last resume packet was sent.
     */
    lastResume: number;
    /**
     * Creates a websocket connection to a Discord gateway.
     *
     * @param {Client} client The client.
     * @param {string} domain The domain to connect to.
     */
    constructor(client: Client, domain: string);
    /**
     * Connect to the gateway.
     *
     * @returns {void} Voids once completed.
     */
    connect(): void;
    /**
     * Reconnect to the gateway.
     *
     * @returns {void} Voids once completed.
     */
    reconnect(): void;
    /**
     * Process a packet.
     *
     * @param {any} data The packet's data.
     * @returns {Promise<void>} Voids once completed.
     */
    private onPacket;
    /**
     * Process the websocket opening.
     *
     * @returns {void} Voids once completed.
     */
    private onOpen;
    /**
     * Process the websocket closing.
     *
     * @param {number} code The closing code.
     * @returns {Promise<void>} Voids once completed.
     */
    private onClose;
    /**
     * Send a packet to the gateway.
     *
     * @param {Object} data The data to send.
     * @returns {void} Voids once sent.
     */
    send(data: Object): void;
    /**
     * Send an identify packet to the gateway.
     *
     * @returns {void} Voids once sent.
     */
    private identify;
    /**
     * Send a heartbeat to the gateway.
     *
     * @returns {Promise<void>} Voids once sent.
     */
    heartbeat(): Promise<void>;
    /**
     * Close the websocket.
     *
     * @param {number} [code] The close code.
     * @returns {void} Voids once completed.
     */
    close(code?: number): void;
    /**
     * Terminate the websocket. Does not notify the gateway
     * that you've disconnected. Should only be used when
     * the gateway won't respond (e.g. timeouts).
     *
     * @returns {void} Voids once completed.
     */
    terminate(): void;
    /**
     * Check if the websocket is connected.
     *
     * @returns {boolean} Whether or not the websocket is connected.
     */
    isConnected(): boolean;
    /**
     * Check if the websocket is connecting.
     *
     * @returns {boolean} Whether or not the websocket is connecting.
     */
    isConnecting(): boolean;
    /**
     * Check if the websocket is closed.
     *
     * @returns {boolean} Whether or not the websocket is closed.
     */
    isClosed(): boolean;
    /**
     * Prevent reconnection.
     *
     * @param {boolean} [doNotReconnect=true] Whether or not to prevent reconnection.
     * @returns {void} Voids once completed.
     */
    preventReconnect(value?: boolean): void;
    /**
     * Set the websocket's status.
     *
     * @param {Status} status The status to set.
     * @returns {void} Voids once completed.
     */
    setStatus(status: Status): void;
    /**
     * Set the websocket's session identifier.
     *
     * @param {string} sessionIdentifier The session identifier to set.
     * @returns {void} Voids once completed.
     */
    setSessionIdentifier(sessionIdentifier: string): void;
    /**
     * Clean up the websocket.
     *
     * @returns {void} Voids once completed.
     */
    private cleanup;
    /**
     * Creates a loop that will manage repeating tasks. It
     * will call it's method every 50ms.
     *
     * @returns {NodeJS.Timer} The NodeJS loop.
     */
    private scheduler;
}
